import React from 'react';
import { Copy, StyledProps } from 'tea-component';

import './text-info.less';
export interface TextInfoItem extends StyledProps {
  label?: React.ReactNode;
  value: string | number;
  theme?: string;
  copyable?: boolean;
  href?: string;
}
export interface TextInfoProps extends StyledProps {
  sourceData: TextInfoItem[];
  dense?: boolean;
  lableBold?: boolean;
}
export function TextInfo({ sourceData, lableBold, className, dense, ...props }: TextInfoProps) {
  return (
    <div className={`text-info-list ${lableBold ? 'text-info-dense' : ''} ${className || ''}`} {...props}>
      {sourceData.map((ele, index) => {
        const { value, label, copyable, className = '', style, href } = ele;
        return value || value === 0 ? (
          <div key={index} className={`text-info-item ${className}`} style={style}>
            <div className="text-info-label txs-h3" style={{ fontWeight: lableBold ? 'bold' : 'normal' }}>
              {label}
            </div>
            <div className="text-info-value">
              {href ? (
                <a className="text-blue" href={href} target="_blank" rel="noreferrer">
                  {value}
                </a>
              ) : (
                <span>{value}</span>
              )}{' '}
              {copyable && value && <Copy text={String(value)} />}
            </div>
          </div>
        ) : null;
      })}
    </div>
  );
}
