import { Chain } from '../utils/interface';
import {
  ACCOUNT_MODE,
  DEFAULT_BROWSER_LINK,
  DEFAULT_HOSTNAME,
  DEFAULT_PK_CHAIN_BROWSER_LINK,
  PROTOCOL_MODE,
} from './chain';

/**
 * 官方内置链,ChainId需保持一致
 */

// node1.chainmaker.org.cn 152.136.217.46
// node2.chainmaker.org.cn 49.232.86.161
// node3.chainmaker.org.cn 82.157.120.56
// node4.chainmaker.org.cn 152.136.210.129

export const OFFICIAL_CHAIN_CERT: Chain[] = [
  {
    accountMode: ACCOUNT_MODE.CERT,
    chainId: 'chainmaker_testnet_chain',
    chainName: '长安链测试网络（Cert）',
    hostName: DEFAULT_HOSTNAME,
    nodeIp: 'certnode1.chainmaker.org.cn:13301',
    nodeTLSCrtFile: null,
    tlsEnable: true,
    userTLSCrtFile: '_official_chain_tls.crt',
    userTLSKeyFile: '_official_chain_tls.key',
    /**
     * 官方链版本信息会在添加用户时进行确定
     */
    version: null,
    proxyURL: CHAIN_MAKER.proxyServerURL2,
    browserLink: DEFAULT_BROWSER_LINK,
  },
  {
    accountMode: ACCOUNT_MODE.CERT,
    chainId: 'chainmaker_testnet_chain',
    chainName: '长安链测试网络（Cert）',
    hostName: DEFAULT_HOSTNAME,
    nodeIp: 'certnode2.chainmaker.org.cn:13302',
    nodeTLSCrtFile: null,
    tlsEnable: true,
    userTLSCrtFile: '_official_chain_tls.crt',
    userTLSKeyFile: '_official_chain_tls.key',
    version: null,
    proxyURL: CHAIN_MAKER.proxyServerURL2,
    browserLink: DEFAULT_BROWSER_LINK,
  },
  {
    accountMode: ACCOUNT_MODE.CERT,
    chainId: 'chainmaker_testnet_chain',
    chainName: '长安链测试网络（Cert）',
    hostName: DEFAULT_HOSTNAME,
    nodeIp: 'certnode3.chainmaker.org.cn:13303',
    nodeTLSCrtFile: null,
    tlsEnable: true,
    userTLSCrtFile: '_official_chain_tls.crt',
    userTLSKeyFile: '_official_chain_tls.key',
    version: null,
    proxyURL: CHAIN_MAKER.proxyServerURL2,
    browserLink: DEFAULT_BROWSER_LINK,
  },
  {
    accountMode: ACCOUNT_MODE.CERT,
    chainId: 'chainmaker_testnet_chain',
    chainName: '长安链测试网络（Cert）',
    hostName: DEFAULT_HOSTNAME,
    nodeIp: 'certnode4.chainmaker.org.cn:13304',
    nodeTLSCrtFile: null,
    tlsEnable: true,
    userTLSCrtFile: '_official_chain_tls.crt',
    userTLSKeyFile: '_official_chain_tls.key',
    version: null,
    proxyURL: CHAIN_MAKER.proxyServerURL2,
    browserLink: DEFAULT_BROWSER_LINK,
  },
];

export const OFFICIAL_CHAIN_PK: Chain[] = [
  {
    accountMode: ACCOUNT_MODE.PK,
    chainId: 'chainmaker_testnet_pk',
    chainName: '长安链测试网络（Public）',
    hostName: DEFAULT_HOSTNAME,
    nodeIp: 'testnode.chainmakernet.com:17301',
    nodeTLSCrtFile: null,
    tlsEnable: false,
    userTLSCrtFile: '',
    userTLSKeyFile: '',
    /**
     * 官方链版本信息会在添加用户时进行确定
     */
    protocol: PROTOCOL_MODE.http,
    httpSupport: true,
    version: null,
    proxyURL: CHAIN_MAKER.proxyServerURL2,
    browserLink: DEFAULT_PK_CHAIN_BROWSER_LINK,
  },
  {
    accountMode: ACCOUNT_MODE.PK,
    chainId: 'chainmaker_testnet_pk',
    chainName: '长安链测试网络（Public）',
    hostName: DEFAULT_HOSTNAME,
    nodeIp: 'testnode.chainmakernet.com:17302',
    nodeTLSCrtFile: null,
    tlsEnable: false,
    userTLSCrtFile: '',
    userTLSKeyFile: '',
    /**
     * 官方链版本信息会在添加用户时进行确定
     */
    protocol: PROTOCOL_MODE.http,
    httpSupport: true,
    version: null,
    proxyURL: CHAIN_MAKER.proxyServerURL2,
    browserLink: DEFAULT_PK_CHAIN_BROWSER_LINK,
  },
  {
    accountMode: ACCOUNT_MODE.PK,
    chainId: 'chainmaker_testnet_pk',
    chainName: '长安链测试网络（Public）',
    hostName: DEFAULT_HOSTNAME,
    nodeIp: 'testnode.chainmakernet.com:17303',
    nodeTLSCrtFile: null,
    tlsEnable: false,
    userTLSCrtFile: '',
    userTLSKeyFile: '',
    /**
     * 官方链版本信息会在添加用户时进行确定
     */
    protocol: PROTOCOL_MODE.http,
    httpSupport: true,
    version: null,
    proxyURL: CHAIN_MAKER.proxyServerURL2,
    browserLink: DEFAULT_PK_CHAIN_BROWSER_LINK,
  },
  {
    accountMode: ACCOUNT_MODE.PK,
    chainId: 'chainmaker_testnet_pk',
    chainName: '长安链测试网络（Public）',
    hostName: DEFAULT_HOSTNAME,
    nodeIp: 'testnode.chainmakernet.com:17304',
    nodeTLSCrtFile: null,
    tlsEnable: false,
    userTLSCrtFile: '',
    userTLSKeyFile: '',
    /**
     * 官方链版本信息会在添加用户时进行确定
     */
    protocol: PROTOCOL_MODE.http,
    httpSupport: true,
    version: null,
    proxyURL: CHAIN_MAKER.proxyServerURL2,
    browserLink: DEFAULT_PK_CHAIN_BROWSER_LINK,
  },
];

export const OFFICIAL_CHAIN_MAP: Record<OfficialChainId, Chain[]> = {
  chainmaker_testnet_chain: OFFICIAL_CHAIN_CERT,
  chainmaker_testnet_pk: OFFICIAL_CHAIN_PK,
};
