/*
 *
 *  Copyright (C) THL A29 Limited, a Tencent company. All rights reserved.
 *  SPDX-License-Identifier: Apache-2.0
 *
 */

import React, {
  useCallback,
} from "react";
import { DetailPage } from "../utils/common";
import { Chain } from "../utils/interface";
import { useLocation, useNavigate } from "react-router-dom";
import AccountNew from "../components/account-new";

function AccountNewPage() {
  const navigate = useNavigate();
  const location = useLocation();
  const { chain, initial } = location.state as {
    chain: Chain;
    initial: boolean;
  };
  const handleCancel = useCallback(() => {
    navigate("/accounts");
  }, []);

  if(!chain) return null;
  return (
    <DetailPage title={"导入链账户"} backUrl={"/accounts"}>
      {initial && (
        <p className="body-tips">
          请导入“{chain.chainName}”的链账户，完成初始化操作。
        </p>
      )}
      <AccountNew
        chain={chain}
        initial={initial}
        onSuccess={() => {
          navigate("/accounts");
        }}
        onCancel={handleCancel}
        onError={handleCancel}
      />
    </DetailPage>
  );
}

export default AccountNewPage;
