/*
 *
 *  Copyright (C) THL A29 Limited, a Tencent company. All rights reserved.
 *  SPDX-License-Identifier: Apache-2.0
 *
 */

import { useNavigate } from 'react-router-dom';
import React, { useCallback, useEffect, useRef, useState } from 'react';
import { Account } from '../utils/interface';
import chainStorageUtils from '../utils/storage';
import { Button, List } from 'tea-component';
import { ConfirmModal, DetailPage, ListContainer } from '../utils/common';
import { useChainStore } from './popup';
import { Copy } from 'tea-component/es';
import { responseAccountInfo } from '../utils/utils';
import { getNowSeconds, sendMessageToContentScript } from '../utils/tools';

function AccountsPage() {
  const navigate = useNavigate();
  const [accounts, setAccounts] = useState<Account[]>([]);
  const {selectedChain,currentAccount,setCurrentAccount,currentTab} = useChainStore();

  const gotoClick = useCallback(() => {
    navigate(`/accounts/new`, {
      state: { 
        chain: selectedChain,
        initial: false
      }
    });
  }, [selectedChain]);
  const deleteConfirmRef = useRef<typeof ConfirmModal>();

  useEffect(() => {
    if (!selectedChain) {
      return;
    }
    chainStorageUtils.getChainAccounts(selectedChain.chainId).then(res => {
      setAccounts(res);
    });
  }, [selectedChain]);

  const handleDeleteChain = useCallback((index: number) => {

    // @ts-ignore
    deleteConfirmRef.current.show({
      confirm: () => chainStorageUtils.deleteChainAccount(selectedChain.chainId, index).then(res => {
        const removedAccounts = accounts[index];
        if(removedAccounts.address===currentAccount.address){
          chainStorageUtils.getCurrentAccount().then(res=>{
            setCurrentAccount(res);
          });
        }
        setAccounts(res);
        const {chainName,chainId} = selectedChain;
        sendMessageToContentScript({
          operation: 'deleteChainAccounts',
          data: {
              status: 'done',
              timestamp: getNowSeconds(),
              info: {
                  removedAccounts:responseAccountInfo([removedAccounts], currentTab?.host),
                  chain: { chainName, chainId }
              }
          },
        });
      }),
    });
  }, [selectedChain,accounts,currentAccount]);


  return <><DetailPage title={'链账户管理'} backUrl={'/'} className="chain-account-manage">
    <ListContainer items={accounts} notFound={'暂无账户'}>
      <List className={'chains mt-2n'}>
        {
          accounts.map((item, index) => {
            return <List.Item key={item.userSignCrtFile} className={'account-item'}>
              <div className={'flex-grow'}>
                <div className={'flex-grow'}>
                  <span className={`${selectedChain.accountMode === 'permissionedWithCert' ? 'avatar-certificate' : 'avatar-public-key'} tea-mr-3n`}>
                    {selectedChain.accountMode === 'permissionedWithCert' ? 'C' : 'P'}
                  </span>
                  {item.name}
                </div>
                {item.address &&
                  <div className={'tea-mt-1n'}>
                    <span className={'tea-mr-1n'}>{item.address}</span>
                    <Copy text={item.address} />
                  </div>
                }
              </div>
              <img src={'/img/icon-delete.png'} onClick={() => handleDeleteChain(index)} title={'删除'} />
            </List.Item>;
          })
        }
      </List>
    </ListContainer>
    <ConfirmModal title={'删除确认'}
      ref={deleteConfirmRef}>
      {'删除链账号后，将无法使用该账号进行交易签名，请确认是否删除该账号。'}
    </ConfirmModal>
  </DetailPage>
    <footer>
      <Button onClick={gotoClick} type={'primary'} className={'btn-lg'}
        disabled={selectedChain === null}>添加链账户</Button>
    </footer>
  </>;
}

export default AccountsPage;
