/*
 *
 *  Copyright (C) THL A29 Limited, a Tencent company. All rights reserved.
 *  SPDX-License-Identifier: Apache-2.0
 *
 */

import React from 'react';
import { useLocation } from 'react-router-dom';
import { Chain } from '../utils/interface';
import { DetailPage, ReadonlyFormItem } from '../utils/common';
import { Form } from 'tea-component';
import { accountModeOptions } from '../utils/utils';
import GlossaryGuide from '../utils/glossary-guide';
import { isOfficialChain } from '../utils/official-chain';

function ChainDetailPage() {
  const location = useLocation();
  const chain = location.state as Chain;
  const isOfficial = isOfficialChain(chain);

  return (
    <DetailPage title={'区块链网络详情'} backUrl={'/chains'}>
      <Form layout={'vertical'}>
        <ReadonlyFormItem label={<GlossaryGuide title={'区块链网络名称'} />} text={chain.chainName} />
        <ReadonlyFormItem label={<GlossaryGuide title={'区块链ID'} />} text={chain.chainId} />
        <ReadonlyFormItem label={'区块链版本'} text={chain.version} />
        {!isOfficial && <ReadonlyFormItem label={<GlossaryGuide title={'节点RPC服务地址'} />} text={chain.nodeIp} />}
        <ReadonlyFormItem label={'是否支持HTTP协议'} text={chain.httpSupport ? '支持' : '不支持'} copyable={false} />
        <ReadonlyFormItem
          label={'账户模式'}
          text={accountModeOptions.find((item) => item.value === chain.accountMode).text}
          copyable={false}
        />
        <ReadonlyFormItem label={'是否开启TLS'} text={chain.tlsEnable ? '开启' : '关闭'} copyable={false} />
        <ReadonlyFormItem label={'TLS hostname'} text={chain.hostName} />
        <ReadonlyFormItem label={'区块链浏览器链接（选填）'} text={chain.browserLink || ''} />
      </Form>
    </DetailPage>
  );
}

export default ChainDetailPage;
